# The code is changed from scikit-learn tutorial:
# https://scikit-learn.org/stable/auto_examples/classification/plot_classifier_comparison.html
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import ListedColormap
from sklearn.datasets import make_moons, make_circles, make_classification
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
from sklearn.gaussian_process import GaussianProcessClassifier
from sklearn.gaussian_process.kernels import RBF
from sklearn.model_selection import train_test_split
from sklearn.naive_bayes import GaussianNB
from sklearn.neural_network import MLPClassifier
from sklearn.preprocessing import StandardScaler
from sklearn.svm import SVC,LinearSVC
from sklearn.tree import DecisionTreeClassifier
import sys
sys.path.append('../')
from aaa_issta import lib

h = .02  # step size in the mesh

names = ["Linear SVM", "RBF SVM", "Gaussian Process",
         "Decision Tree", "Random Forest", "AdaBoost",
         "Naive Bayes"]

classifiers = [
    SVC(kernel="linear",C=0.025),
    SVC(gamma=2, C=1),
    GaussianProcessClassifier(1.0 * RBF(1.0)),
    DecisionTreeClassifier(max_depth=10),
    RandomForestClassifier(max_depth=10, n_estimators=10, max_features=1),
    AdaBoostClassifier(),
    GaussianNB(),
]

label_noide_degree = 0.2

num_points = 2100 # total number of data points; 100 are training data points, 2000 are hold-out test data points

# generate linearly-separable data
def make_linear_separable(num_points, originalnoise):
    if label_noide_degree>0:
        flip = originalnoise
    else:
        flip = -1
    X, y = make_classification(n_samples=num_points, n_features=2, n_redundant=0, n_informative=1,random_state=42,
                                n_clusters_per_class=1, flip_y=flip)
    rng = np.random.RandomState(2)
    X += 2 * rng.uniform(size=X.shape)
    return (X, y)


# three known distribution with a particular label noise degree
datasets = [
    make_moons(noise=label_noide_degree, random_state=0, n_samples=num_points),
    make_circles(noise=label_noide_degree, factor=0.5, random_state=42, n_samples=num_points),
    make_linear_separable(num_points, label_noide_degree),

]

Xlist = []
Ylist = []
xtestlist = []
ytestlist = []

for each in datasets:
    X0,Y0 = each
    X0 = StandardScaler().fit_transform(X0)
    Xtrain, X_test, Ytrain, Y_test = train_test_split(X0, Y0, test_size=2000, random_state=42,shuffle=True) # get 2000 test data points
    Xlist.append(Xtrain)
    Ylist.append(Ytrain)
    xtestlist.append(X_test)
    ytestlist.append(Y_test)


def draw_Figure2():
    plt.figure(figsize=(12,4.9))
    i = 1

    # iterate over the three datasets
    for datano in np.arange(0,len(datasets),1):


        X_train = Xlist[datano]
        X_test = xtestlist[datano]
        y_train = Ylist[datano]
        y_test = ytestlist[datano]


        x_min, x_max = X_train[:, 0].min() - .5, X_train[:, 0].max() + .5
        y_min, y_max = X_train[:, 1].min() - .5, X_train[:, 1].max() + .5

        xx, yy = np.meshgrid(np.arange(x_min, x_max, h),
                             np.arange(y_min, y_max, h))

        # just plot the dataset first
        cm = plt.cm.RdBu
        cm_bright = ListedColormap(['#FF0000', '#0000FF'])
        ax = plt.subplot(3, len(classifiers) + 1, i)
        if datano == 0:
            if label_noide_degree <0.2:
                ax.set_title("zero noise")
            else:
                ax.set_title(str(round(label_noide_degree,1))+" noise")

        # plot the training points
        ax.scatter(X_train[:, 0], X_train[:, 1], c=y_train, s=10, cmap=cm_bright,
                   edgecolors='k')
        ax.set_xlim(xx.min(), xx.max())
        ax.set_ylim(yy.min(), yy.max())
        ax.set_xticks(())
        ax.set_yticks(())
        i += 1

        # iterate over classifiers
        for name, clf in zip(names, classifiers):
            print('-------')
            print(name)
            ax = plt.subplot(3, len(classifiers) + 1, i)
            y_train = lib.get_noisydata_labelperturbation(y_train, 0.0)


            # get metric values
            dic_metric_value = lib.get_allmetrics_classic(clf, X_train,y_train,X_test,y_test)

            pv = dic_metric_value['pv']
            cv = dic_metric_value['cv']
            testaccuracy = dic_metric_value['test']

            clf.fit(X_train,y_train) # use the model trained on the original training data to draw decision boundaries

            # Plot the decision boundary. For that, we will assign a color to each
            # point in the mesh [x_min, x_max]x[y_min, y_max].
            if hasattr(clf, "decision_function"):
                Z = clf.decision_function(np.c_[xx.ravel(), yy.ravel()])
            else:
                Z = clf.predict_proba(np.c_[xx.ravel(), yy.ravel()])[:, 1]

            # Put the result into a color plot
            Z = Z.reshape(xx.shape)
            ax.contourf(xx, yy, Z, cmap=cm, alpha=.65)

            # Plot also the training points
            ax.scatter(X_train[:, 0], X_train[:, 1], c=y_train, s=10, cmap=cm_bright,
                       edgecolors='k',alpha = 0.8)

            ax.set_xlim(xx.min(), xx.max())
            ax.set_ylim(yy.min(), yy.max())
            ax.set_xticks(())
            ax.set_yticks(())
            if datano == 0:
                ax.set_title(name)


            textstr = '\n'.join((
               r'MV=%.2f' % (pv,),
               r'CV=%.2f' % (cv,),
               r'Test=%.2f' % (testaccuracy,),

            ))
            props = dict(boxstyle='round', facecolor='white', alpha=0.65)
            ax.text(0.05, 0.95, textstr, transform=ax.transAxes, fontsize=10,
                    verticalalignment='top', bbox=props)

            i += 1

    plt.tight_layout()
    plt.subplots_adjust(wspace=0.06, hspace=0.06)
    plt.savefig("./../plots/2noise.pdf", bbox_inches='tight')
    plt.show()



if __name__ == '__main__':
  draw_Figure2()
